@@ -9,12 +9,12 @@ use super::{
 use crate::platform_impl::platform::x11::ime::ImeEventSender;
 
 pub(crate) unsafe fn close_im(xconn: &Arc<XConnection>, im: ffi::XIM) -> Result<(), XError> {
-    (xconn.xlib.XCloseIM)(im);
+    unsafe { (xconn.xlib.XCloseIM)(im) };
     xconn.check_errors()
 }
 
 pub(crate) unsafe fn destroy_ic(xconn: &Arc<XConnection>, ic: ffi::XIC) -> Result<(), XError> {
-    (xconn.xlib.XDestroyIC)(ic);
+    unsafe { (xconn.xlib.XDestroyIC)(ic) };
     xconn.check_errors()
 }
 
@@ -52,7 +52,7 @@ impl ImeInner {
 
     pub unsafe fn close_im_if_necessary(&self) -> Result<bool, XError> {
         if !self.is_destroyed && self.im.is_some() {
-            close_im(&self.xconn, self.im.as_ref().unwrap().im).map(|_| true)
+            unsafe { close_im(&self.xconn, self.im.as_ref().unwrap().im) }.map(|_| true)
         } else {
             Ok(false)
         }
@@ -60,7 +60,7 @@ impl ImeInner {
 
     pub unsafe fn destroy_ic_if_necessary(&self, ic: ffi::XIC) -> Result<bool, XError> {
         if !self.is_destroyed {
-            destroy_ic(&self.xconn, ic).map(|_| true)
+            unsafe { destroy_ic(&self.xconn, ic) }.map(|_| true)
         } else {
             Ok(false)
         }
@@ -68,7 +68,7 @@ impl ImeInner {
 
     pub unsafe fn destroy_all_contexts_if_necessary(&self) -> Result<bool, XError> {
         for context in self.contexts.values().flatten() {
-            self.destroy_ic_if_necessary(context.ic)?;
+            unsafe { self.destroy_ic_if_necessary(context.ic)? };
         }
         Ok(!self.is_destroyed)
     }
