@@ -1,6 +1,6 @@
-use objc2::foundation::{NSData, NSObject, NSString};
-use objc2::rc::{Id, Shared};
-use objc2::{extern_class, extern_methods, msg_send_id, ClassType};
+use icrate::Foundation::{NSData, NSObject, NSString};
+use objc2::rc::Id;
+use objc2::{extern_class, extern_methods, msg_send_id, mutability, ClassType};
 
 extern_class!(
     // TODO: Can this be mutable?
@@ -9,6 +9,7 @@ extern_class!(
 
     unsafe impl ClassType for NSImage {
         type Super = NSObject;
+        type Mutability = mutability::InteriorMutable;
     }
 );
 
@@ -24,14 +25,12 @@ unsafe impl Sync for NSImage {}
 
 extern_methods!(
     unsafe impl NSImage {
-        pub fn new_by_referencing_file(path: &NSString) -> Id<Self, Shared> {
-            let this = unsafe { msg_send_id![Self::class(), alloc] };
-            unsafe { msg_send_id![this, initByReferencingFile: path] }
+        pub fn new_by_referencing_file(path: &NSString) -> Id<Self> {
+            unsafe { msg_send_id![Self::alloc(), initByReferencingFile: path] }
         }
 
-        pub fn new_with_data(data: &NSData) -> Id<Self, Shared> {
-            let this = unsafe { msg_send_id![Self::class(), alloc] };
-            unsafe { msg_send_id![this, initWithData: data] }
+        pub fn new_with_data(data: &NSData) -> Id<Self> {
+            unsafe { msg_send_id![Self::alloc(), initWithData: data] }
         }
     }
 );
