@@ -1,6 +1,6 @@
-use objc2::foundation::{NSObject, NSString};
-use objc2::rc::{Id, Shared};
-use objc2::{extern_class, extern_methods, msg_send_id, ClassType};
+use icrate::Foundation::{NSObject, NSString};
+use objc2::rc::Id;
+use objc2::{extern_class, extern_methods, mutability, ClassType};
 
 type NSTextInputSourceIdentifier = NSString;
 
@@ -11,21 +11,19 @@ extern_class!(
 
     unsafe impl ClassType for NSTextInputContext {
         type Super = NSObject;
+        type Mutability = mutability::InteriorMutable;
     }
 );
 
 extern_methods!(
     unsafe impl NSTextInputContext {
-        #[sel(invalidateCharacterCoordinates)]
+        #[method(invalidateCharacterCoordinates)]
         pub fn invalidateCharacterCoordinates(&self);
 
-        #[sel(discardMarkedText)]
+        #[method(discardMarkedText)]
         pub fn discardMarkedText(&self);
 
-        pub fn selectedKeyboardInputSource(
-            &self,
-        ) -> Option<Id<NSTextInputSourceIdentifier, Shared>> {
-            unsafe { msg_send_id![self, selectedKeyboardInputSource] }
-        }
+        #[method_id(selectedKeyboardInputSource)]
+        pub fn selectedKeyboardInputSource(&self) -> Option<Id<NSTextInputSourceIdentifier>>;
     }
 );
