@@ -1,68 +1,80 @@
 #![allow(clippy::unnecessary_cast)]
+use std::cell::Cell;
+use std::ptr::{self, NonNull};
 
-use std::ptr;
-
+use icrate::Foundation::{NSArray, NSObject, NSSize, NSString};
 use objc2::declare::{Ivar, IvarDrop};
-use objc2::foundation::{NSArray, NSObject, NSSize, NSString};
-use objc2::rc::{autoreleasepool, Id, Shared};
-use objc2::runtime::Object;
-use objc2::{class, declare_class, msg_send, msg_send_id, sel, ClassType};
+use objc2::rc::{autoreleasepool, Id};
+use objc2::runtime::AnyObject;
+use objc2::{class, declare_class, msg_send, msg_send_id, mutability, sel, ClassType};
 
 use super::appkit::{
     NSApplicationPresentationOptions, NSFilenamesPboardType, NSPasteboard, NSWindowOcclusionState,
 };
+use super::{
+    app_state::AppState,
+    util,
+    window::{get_ns_theme, WinitWindow},
+    Fullscreen,
+};
 use crate::{
     dpi::{LogicalPosition, LogicalSize},
-    event::{Event, ModifiersState, WindowEvent},
-    platform_impl::platform::{
-        app_state::AppState,
-        event::{EventProxy, EventWrapper},
-        util,
-        window::{get_ns_theme, WinitWindow},
-        Fullscreen,
-    },
+    event::{Event, WindowEvent},
     window::WindowId,
 };
 
+#[derive(Debug)]
+pub struct State {
+    // This is set when WindowBuilder::with_fullscreen was set,
+    // see comments of `window_did_fail_to_enter_fullscreen`
+    initial_fullscreen: Cell<bool>,
+
+    // During `windowDidResize`, we use this to only send Moved if the position changed.
+    previous_position: Cell<Option<(f64, f64)>>,
+
+    // Used to prevent redundant events.
+    previous_scale_factor: Cell<f64>,
+}
+
 declare_class!(
     #[derive(Debug)]
     pub(crate) struct WinitWindowDelegate {
-        window: IvarDrop<Id<WinitWindow, Shared>>,
+        window: IvarDrop<Id<WinitWindow>, "_window">,
 
-        // TODO: It's possible for delegate methods to be called asynchronously,
-        // causing data races / `RefCell` panics.
-
-        // This is set when WindowBuilder::with_fullscreen was set,
-        // see comments of `window_did_fail_to_enter_fullscreen`
-        initial_fullscreen: bool,
-
-        // During `windowDidResize`, we use this to only send Moved if the position changed.
+        // TODO: It may be possible for delegate methods to be called
+        // asynchronously, causing data races panics?
         // TODO: Remove unnecessary boxing here
-        previous_position: IvarDrop<Option<Box<(f64, f64)>>>,
-
-        // Used to prevent redundant events.
-        previous_scale_factor: f64,
+        state: IvarDrop<Box<State>, "_state">,
     }
 
+    mod ivars;
+
     unsafe impl ClassType for WinitWindowDelegate {
         type Super = NSObject;
+        type Mutability = mutability::InteriorMutable;
+        const NAME: &'static str = "WinitWindowDelegate";
     }
 
     unsafe impl WinitWindowDelegate {
-        #[sel(initWithWindow:initialFullscreen:)]
-        fn init_with_winit(
-            &mut self,
+        #[method(initWithWindow:initialFullscreen:)]
+        unsafe fn init_with_winit(
+            this: *mut Self,
             window: &WinitWindow,
             initial_fullscreen: bool,
-        ) -> Option<&mut Self> {
-            let this: Option<&mut Self> = unsafe { msg_send![self, init] };
+        ) -> Option<NonNull<Self>> {
+            let this: Option<&mut Self> = unsafe { msg_send![super(this), init] };
             this.map(|this| {
                 let scale_factor = window.scale_factor();
 
                 Ivar::write(&mut this.window, window.retain());
-                Ivar::write(&mut this.initial_fullscreen, initial_fullscreen);
-                Ivar::write(&mut this.previous_position, None);
-                Ivar::write(&mut this.previous_scale_factor, scale_factor);
+                Ivar::write(
+                    &mut this.state,
+                    Box::new(State {
+                        initial_fullscreen: Cell::new(initial_fullscreen),
+                        previous_position: Cell::new(None),
+                        previous_scale_factor: Cell::new(scale_factor),
+                    }),
+                );
 
                 if scale_factor != 1.0 {
                     this.queue_static_scale_factor_changed_event();
@@ -70,7 +82,7 @@ declare_class!(
                 this.window.setDelegate(Some(this));
 
                 // Enable theme change event
-                let notification_center: Id<Object, Shared> =
+                let notification_center: Id<AnyObject> =
                     unsafe { msg_send_id![class!(NSDistributedNotificationCenter), defaultCenter] };
                 let notification_name =
                     NSString::from_str("AppleInterfaceThemeChangedNotification");
@@ -80,26 +92,26 @@ declare_class!(
                         addObserver: &*this
                         selector: sel!(effectiveAppearanceDidChange:)
                         name: &*notification_name
-                        object: ptr::null::<Object>()
+                        object: ptr::null::<AnyObject>()
                     ]
                 };
 
-                this
+                NonNull::from(this)
             })
         }
     }
 
     // NSWindowDelegate + NSDraggingDestination protocols
     unsafe impl WinitWindowDelegate {
-        #[sel(windowShouldClose:)]
-        fn window_should_close(&self, _: Option<&Object>) -> bool {
+        #[method(windowShouldClose:)]
+        fn window_should_close(&self, _: Option<&AnyObject>) -> bool {
             trace_scope!("windowShouldClose:");
             self.queue_event(WindowEvent::CloseRequested);
             false
         }
 
-        #[sel(windowWillClose:)]
-        fn window_will_close(&self, _: Option<&Object>) {
+        #[method(windowWillClose:)]
+        fn window_will_close(&self, _: Option<&AnyObject>) {
             trace_scope!("windowWillClose:");
             // `setDelegate:` retains the previous value and then autoreleases it
             autoreleasepool(|_| {
@@ -110,15 +122,15 @@ declare_class!(
             self.queue_event(WindowEvent::Destroyed);
         }
 
-        #[sel(windowDidResize:)]
-        fn window_did_resize(&mut self, _: Option<&Object>) {
+        #[method(windowDidResize:)]
+        fn window_did_resize(&self, _: Option<&AnyObject>) {
             trace_scope!("windowDidResize:");
             // NOTE: WindowEvent::Resized is reported in frameDidChange.
             self.emit_move_event();
         }
 
-        #[sel(windowWillStartLiveResize:)]
-        fn window_will_start_live_resize(&mut self, _: Option<&Object>) {
+        #[method(windowWillStartLiveResize:)]
+        fn window_will_start_live_resize(&self, _: Option<&AnyObject>) {
             trace_scope!("windowWillStartLiveResize:");
 
             let increments = self
@@ -128,35 +140,35 @@ declare_class!(
             self.window.set_resize_increments_inner(increments);
         }
 
-        #[sel(windowDidEndLiveResize:)]
-        fn window_did_end_live_resize(&mut self, _: Option<&Object>) {
+        #[method(windowDidEndLiveResize:)]
+        fn window_did_end_live_resize(&self, _: Option<&AnyObject>) {
             trace_scope!("windowDidEndLiveResize:");
             self.window.set_resize_increments_inner(NSSize::new(1., 1.));
         }
 
         // This won't be triggered if the move was part of a resize.
-        #[sel(windowDidMove:)]
-        fn window_did_move(&mut self, _: Option<&Object>) {
+        #[method(windowDidMove:)]
+        fn window_did_move(&self, _: Option<&AnyObject>) {
             trace_scope!("windowDidMove:");
             self.emit_move_event();
         }
 
-        #[sel(windowDidChangeBackingProperties:)]
-        fn window_did_change_backing_properties(&mut self, _: Option<&Object>) {
+        #[method(windowDidChangeBackingProperties:)]
+        fn window_did_change_backing_properties(&self, _: Option<&AnyObject>) {
             trace_scope!("windowDidChangeBackingProperties:");
             self.queue_static_scale_factor_changed_event();
         }
 
-        #[sel(windowDidBecomeKey:)]
-        fn window_did_become_key(&self, _: Option<&Object>) {
+        #[method(windowDidBecomeKey:)]
+        fn window_did_become_key(&self, _: Option<&AnyObject>) {
             trace_scope!("windowDidBecomeKey:");
             // TODO: center the cursor if the window had mouse grab when it
             // lost focus
             self.queue_event(WindowEvent::Focused(true));
         }
 
-        #[sel(windowDidResignKey:)]
-        fn window_did_resign_key(&self, _: Option<&Object>) {
+        #[method(windowDidResignKey:)]
+        fn window_did_resign_key(&self, _: Option<&AnyObject>) {
             trace_scope!("windowDidResignKey:");
             // It happens rather often, e.g. when the user is Cmd+Tabbing, that the
             // NSWindowDelegate will receive a didResignKey event despite no event
@@ -165,29 +177,21 @@ declare_class!(
             // NSWindowDelegate, and as a result a tracked modifiers state can quite
             // easily fall out of synchrony with reality.  This requires us to emit
             // a synthetic ModifiersChanged event when we lose focus.
-
-            // TODO(madsmtm): Remove the need for this unsafety
-            let mut view = unsafe { Id::from_shared(self.window.view()) };
-
-            // Both update the state and emit a ModifiersChanged event.
-            if !view.state.modifiers.is_empty() {
-                view.state.modifiers = ModifiersState::empty();
-                self.queue_event(WindowEvent::ModifiersChanged(view.state.modifiers));
-            }
+            self.window.view().reset_modifiers();
 
             self.queue_event(WindowEvent::Focused(false));
         }
 
         /// Invoked when the dragged image enters destination bounds or frame
-        #[sel(draggingEntered:)]
-        fn dragging_entered(&self, sender: *mut Object) -> bool {
+        #[method(draggingEntered:)]
+        fn dragging_entered(&self, sender: &NSObject) -> bool {
             trace_scope!("draggingEntered:");
 
             use std::path::PathBuf;
 
-            let pb: Id<NSPasteboard, Shared> = unsafe { msg_send_id![sender, draggingPasteboard] };
+            let pb: Id<NSPasteboard> = unsafe { msg_send_id![sender, draggingPasteboard] };
             let filenames = pb.propertyListForType(unsafe { NSFilenamesPboardType });
-            let filenames: Id<NSArray<NSString>, Shared> = unsafe { Id::cast(filenames) };
+            let filenames: Id<NSArray<NSString>> = unsafe { Id::cast(filenames) };
 
             filenames.into_iter().for_each(|file| {
                 let path = PathBuf::from(file.to_string());
@@ -198,22 +202,22 @@ declare_class!(
         }
 
         /// Invoked when the image is released
-        #[sel(prepareForDragOperation:)]
-        fn prepare_for_drag_operation(&self, _: Option<&Object>) -> bool {
+        #[method(prepareForDragOperation:)]
+        fn prepare_for_drag_operation(&self, _sender: &NSObject) -> bool {
             trace_scope!("prepareForDragOperation:");
             true
         }
 
         /// Invoked after the released image has been removed from the screen
-        #[sel(performDragOperation:)]
-        fn perform_drag_operation(&self, sender: *mut Object) -> bool {
+        #[method(performDragOperation:)]
+        fn perform_drag_operation(&self, sender: &NSObject) -> bool {
             trace_scope!("performDragOperation:");
 
             use std::path::PathBuf;
 
-            let pb: Id<NSPasteboard, Shared> = unsafe { msg_send_id![sender, draggingPasteboard] };
+            let pb: Id<NSPasteboard> = unsafe { msg_send_id![sender, draggingPasteboard] };
             let filenames = pb.propertyListForType(unsafe { NSFilenamesPboardType });
-            let filenames: Id<NSArray<NSString>, Shared> = unsafe { Id::cast(filenames) };
+            let filenames: Id<NSArray<NSString>> = unsafe { Id::cast(filenames) };
 
             filenames.into_iter().for_each(|file| {
                 let path = PathBuf::from(file.to_string());
@@ -224,21 +228,21 @@ declare_class!(
         }
 
         /// Invoked when the dragging operation is complete
-        #[sel(concludeDragOperation:)]
-        fn conclude_drag_operation(&self, _: Option<&Object>) {
+        #[method(concludeDragOperation:)]
+        fn conclude_drag_operation(&self, _sender: Option<&NSObject>) {
             trace_scope!("concludeDragOperation:");
         }
 
         /// Invoked when the dragging operation is cancelled
-        #[sel(draggingExited:)]
-        fn dragging_exited(&self, _: Option<&Object>) {
+        #[method(draggingExited:)]
+        fn dragging_exited(&self, _sender: Option<&NSObject>) {
             trace_scope!("draggingExited:");
             self.queue_event(WindowEvent::HoveredFileCancelled);
         }
 
         /// Invoked when before enter fullscreen
-        #[sel(windowWillEnterFullScreen:)]
-        fn window_will_enter_fullscreen(&self, _: Option<&Object>) {
+        #[method(windowWillEnterFullScreen:)]
+        fn window_will_enter_fullscreen(&self, _: Option<&AnyObject>) {
             trace_scope!("windowWillEnterFullScreen:");
 
             let mut shared_state = self
@@ -266,18 +270,18 @@ declare_class!(
         }
 
         /// Invoked when before exit fullscreen
-        #[sel(windowWillExitFullScreen:)]
-        fn window_will_exit_fullscreen(&self, _: Option<&Object>) {
+        #[method(windowWillExitFullScreen:)]
+        fn window_will_exit_fullscreen(&self, _: Option<&AnyObject>) {
             trace_scope!("windowWillExitFullScreen:");
 
             let mut shared_state = self.window.lock_shared_state("window_will_exit_fullscreen");
             shared_state.in_fullscreen_transition = true;
         }
 
-        #[sel(window:willUseFullScreenPresentationOptions:)]
+        #[method(window:willUseFullScreenPresentationOptions:)]
         fn window_will_use_fullscreen_presentation_options(
             &self,
-            _: Option<&Object>,
+            _: Option<&AnyObject>,
             proposed_options: NSApplicationPresentationOptions,
         ) -> NSApplicationPresentationOptions {
             trace_scope!("window:willUseFullScreenPresentationOptions:");
@@ -303,10 +307,10 @@ declare_class!(
         }
 
         /// Invoked when entered fullscreen
-        #[sel(windowDidEnterFullScreen:)]
-        fn window_did_enter_fullscreen(&mut self, _: Option<&Object>) {
+        #[method(windowDidEnterFullScreen:)]
+        fn window_did_enter_fullscreen(&self, _: Option<&AnyObject>) {
             trace_scope!("windowDidEnterFullScreen:");
-            *self.initial_fullscreen = false;
+            self.state.initial_fullscreen.set(false);
             let mut shared_state = self.window.lock_shared_state("window_did_enter_fullscreen");
             shared_state.in_fullscreen_transition = false;
             let target_fullscreen = shared_state.target_fullscreen.take();
@@ -317,8 +321,8 @@ declare_class!(
         }
 
         /// Invoked when exited fullscreen
-        #[sel(windowDidExitFullScreen:)]
-        fn window_did_exit_fullscreen(&self, _: Option<&Object>) {
+        #[method(windowDidExitFullScreen:)]
+        fn window_did_exit_fullscreen(&self, _: Option<&AnyObject>) {
             trace_scope!("windowDidExitFullScreen:");
 
             self.window.restore_state_from_fullscreen();
@@ -347,21 +351,21 @@ declare_class!(
         /// due to being in the midst of handling some other animation or user gesture.
         /// This method indicates that there was an error, and you should clean up any
         /// work you may have done to prepare to enter full-screen mode.
-        #[sel(windowDidFailToEnterFullScreen:)]
-        fn window_did_fail_to_enter_fullscreen(&self, _: Option<&Object>) {
+        #[method(windowDidFailToEnterFullScreen:)]
+        fn window_did_fail_to_enter_fullscreen(&self, _: Option<&AnyObject>) {
             trace_scope!("windowDidFailToEnterFullScreen:");
             let mut shared_state = self
                 .window
                 .lock_shared_state("window_did_fail_to_enter_fullscreen");
             shared_state.in_fullscreen_transition = false;
             shared_state.target_fullscreen = None;
-            if *self.initial_fullscreen {
+            if self.state.initial_fullscreen.get() {
                 #[allow(clippy::let_unit_value)]
                 unsafe {
                     let _: () = msg_send![
                         &*self.window,
                         performSelector: sel!(toggleFullScreen:),
-                        withObject: ptr::null::<Object>(),
+                        withObject: ptr::null::<AnyObject>(),
                         afterDelay: 0.5,
                     ];
                 };
@@ -371,8 +375,8 @@ declare_class!(
         }
 
         // Invoked when the occlusion state of the window changes
-        #[sel(windowDidChangeOcclusionState:)]
-        fn window_did_change_occlusion_state(&self, _: Option<&Object>) {
+        #[method(windowDidChangeOcclusionState:)]
+        fn window_did_change_occlusion_state(&self, _: Option<&AnyObject>) {
             trace_scope!("windowDidChangeOcclusionState:");
             self.queue_event(WindowEvent::Occluded(
                 !self
@@ -383,8 +387,8 @@ declare_class!(
         }
 
         // Observe theme change
-        #[sel(effectiveAppearanceDidChange:)]
-        fn effective_appearance_did_change(&self, sender: Option<&Object>) {
+        #[method(effectiveAppearanceDidChange:)]
+        fn effective_appearance_did_change(&self, sender: Option<&AnyObject>) {
             trace_scope!("Triggered `effectiveAppearanceDidChange:`");
             unsafe {
                 msg_send![
@@ -396,8 +400,8 @@ declare_class!(
             }
         }
 
-        #[sel(effectiveAppearanceDidChangedOnMainThread:)]
-        fn effective_appearance_did_changed_on_main_thread(&self, _: Option<&Object>) {
+        #[method(effectiveAppearanceDidChangedOnMainThread:)]
+        fn effective_appearance_did_changed_on_main_thread(&self, _: Option<&AnyObject>) {
             let theme = get_ns_theme();
             let mut shared_state = self
                 .window
@@ -410,8 +414,8 @@ declare_class!(
             }
         }
 
-        #[sel(windowDidChangeScreen:)]
-        fn window_did_change_screen(&self, _: Option<&Object>) {
+        #[method(windowDidChangeScreen:)]
+        fn window_did_change_screen(&self, _: Option<&AnyObject>) {
             trace_scope!("windowDidChangeScreen:");
             let is_simple_fullscreen = self
                 .window
@@ -427,45 +431,45 @@ declare_class!(
 );
 
 impl WinitWindowDelegate {
-    pub fn new(window: &WinitWindow, initial_fullscreen: bool) -> Id<Self, Shared> {
+    pub fn new(window: &WinitWindow, initial_fullscreen: bool) -> Id<Self> {
         unsafe {
             msg_send_id![
-                msg_send_id![Self::class(), alloc],
+                Self::alloc(),
                 initWithWindow: window,
                 initialFullscreen: initial_fullscreen,
             ]
         }
     }
 
-    pub(crate) fn queue_event(&self, event: WindowEvent<'static>) {
+    pub(crate) fn queue_event(&self, event: WindowEvent) {
         let event = Event::WindowEvent {
             window_id: WindowId(self.window.id()),
             event,
         };
-        AppState::queue_event(EventWrapper::StaticEvent(event));
+        AppState::queue_event(event);
     }
 
-    fn queue_static_scale_factor_changed_event(&mut self) {
+    fn queue_static_scale_factor_changed_event(&self) {
         let scale_factor = self.window.scale_factor();
-        if scale_factor == *self.previous_scale_factor {
+        if scale_factor == self.state.previous_scale_factor.get() {
             return;
         };
 
-        *self.previous_scale_factor = scale_factor;
-        let wrapper = EventWrapper::EventProxy(EventProxy::DpiChangedProxy {
-            window: self.window.clone(),
-            suggested_size: self.view_size(),
+        self.state.previous_scale_factor.set(scale_factor);
+        let suggested_size = self.view_size();
+        AppState::queue_static_scale_factor_changed_event(
+            self.window.clone(),
+            suggested_size.to_physical(scale_factor),
             scale_factor,
-        });
-        AppState::queue_event(wrapper);
+        );
     }
 
-    fn emit_move_event(&mut self) {
+    fn emit_move_event(&self) {
         let rect = self.window.frame();
         let x = rect.origin.x as f64;
         let y = util::bottom_left_to_top_left(rect);
-        if self.previous_position.as_deref() != Some(&(x, y)) {
-            *self.previous_position = Some(Box::new((x, y)));
+        if self.state.previous_position.get() != Some((x, y)) {
+            self.state.previous_position.set(Some((x, y)));
             let scale_factor = self.window.scale_factor();
             let physical_pos = LogicalPosition::<f64>::from((x, y)).to_physical(scale_factor);
             self.queue_event(WindowEvent::Moved(physical_pos));
