@@ -3,10 +3,8 @@
 use serde::{Deserialize, Serialize};
 use winit::{
     dpi::{LogicalPosition, LogicalSize, PhysicalPosition, PhysicalSize},
-    event::{
-        ElementState, KeyboardInput, ModifiersState, MouseButton, MouseScrollDelta, TouchPhase,
-        VirtualKeyCode,
-    },
+    event::{ElementState, MouseButton, MouseScrollDelta, TouchPhase},
+    keyboard::{Key, KeyCode, KeyLocation, ModifiersState, NamedKey, PhysicalKey},
     window::CursorIcon,
 };
 
@@ -20,12 +18,15 @@ fn window_serde() {
 
 #[test]
 fn events_serde() {
-    needs_serde::<KeyboardInput>();
     needs_serde::<TouchPhase>();
     needs_serde::<ElementState>();
     needs_serde::<MouseButton>();
     needs_serde::<MouseScrollDelta>();
-    needs_serde::<VirtualKeyCode>();
+    needs_serde::<Key>();
+    needs_serde::<NamedKey>();
+    needs_serde::<KeyCode>();
+    needs_serde::<PhysicalKey>();
+    needs_serde::<KeyLocation>();
     needs_serde::<ModifiersState>();
 }
 
